﻿using System.Collections.Generic;
using DataLayer;
using NUnit.Framework;
using Rhino.Mocks;
using System.Linq;

namespace BusinessLayer.Tests
{
    [TestFixture]
    public class BugControllerTests
    {
        private MockRepository _mocks;
        private IRepository<Bug> _repository;
        private StubEmailService _emailStub;
        private BugController _controller;
        private Bug bug;
        private List<Bug> bugs;

        [SetUp]
        public void Setup()
        {
            _mocks = new MockRepository();
            _repository = _mocks.CreateMock<IRepository<Bug>>();

            _emailStub = new StubEmailService();


            _controller = new BugController(_repository, _emailStub);
            bug = CreateBug();
            bugs = new List<Bug> {bug};
        }

        [TearDown]
        public void TearDown()
        {
            _mocks = null;
            _repository = null;
            _emailStub = null;
            _controller = null;
        }

        private static Bug CreateBug()
        {
            Bug b = new Bug
                        {
                            User = new User
                                       {
                                           username = "sa"
                                       },
                            ID = 1,
                            Title = "Cannot insert bug"
                        };
            return b;
        }

        private Bug CreateInvalidBug()
        {
            Bug b = new Bug
                        {
                            Title = string.Empty
                        };
            
            return b;
        }

        [Test]
        public void InsertBug_NewBug_ReturnTrue()
        {
            Bug bug = new Bug();
            
            using (_mocks.Record())
                Expect.Call(_repository.Add(bug)).Return(true);

            using (_mocks.Playback())
            {
                bool inserted = _controller.InsertBug(bug);
                Assert.IsTrue(inserted, "Bug not inserted");
            }
        }

        [Test]
        public void InsertBug_Null_ReturnFalse()
        {
            BugController ctrl = new BugController(null, null);
            bool inserted = ctrl.InsertBug(null);
            Assert.IsFalse(inserted, "Bug was inserted");
        }

        [Test]
        public void InsertBug_BugWithoutTitle_CannotBeInserted_ReturnsFalse()
        {
            Bug b = new Bug();
            b.Title = string.Empty;

            BugController ctrl = new BugController(null, null);
            bool inserted = ctrl.InsertBug(b);
            Assert.IsFalse(inserted, "Bug was inserted");         
        }

        [Test]
        public void IsBugValid_NullObject_ReturnFalse()
        {
            bool valid = _controller.IsBugValid(null);

            Assert.IsFalse(valid, "Bug was marked as valid");
        }

        [Test]
        public void IsBugValid_TitleIsEmpty_ReturnFalse()
        {
            Bug b = CreateInvalidBug();
            bool valid = _controller.IsBugValid(b);

            Assert.IsFalse(valid, "Bug was marked as valid");
        }


        [Test]
        public void IsBugValid_BugWithTitle_ReturnsTrue()
        {
            Bug b = CreateBug();
            bool valid = _controller.IsBugValid(b);

            Assert.IsTrue(valid, "Bug was marked as invalid");
        }

        [Test]
        public void GetBug_ValidID_ReturnsBugObject()
        {
            using (_mocks.Record())
                Expect.Call(_repository.Find(b => b.ID == 1)).IgnoreArguments().Return(bug);

            using (_mocks.Playback())
            {
                Bug actualBug = _controller.GetBug(1);
                Assert.AreEqual(bug, actualBug, "Bug objects are not the same");
            }
        }

        [Test]
        public void GetBug_ValidID_AssignedToSa()
        {
            using (_mocks.Record())
                Expect.Call(_repository.Find(b => b.ID == 1)).IgnoreArguments().Return(bug);

            using (_mocks.Playback())
            {
                Bug actualBug = _controller.GetBug(1);
                Assert.AreEqual("sa", actualBug.User.username);
            }
        }

        [Test]
        public void GetBug_NonExistentID_ReturnsNull()
        {
            using (_mocks.Record())
                Expect.Call(_repository.Find(b => b.ID == 1)).IgnoreArguments().Return(null);

            using (_mocks.Playback())
            {
                Bug bug = _controller.GetBug(0);
                Assert.IsNull(bug, "Bug object not null");
            }
        }

        [Test]
        public void GetAllActiveBugs_NoArgs_ReturnsBugWithinCollection()
        {
            using (_mocks.Record())
                Expect.Call(_repository
                            .Where(b => b.IsActive))
                            .IgnoreArguments()
                            .Return(bugs.AsQueryable());

            using (_mocks.Playback())
            {
                List<Bug> actualBugs = _controller.GetAllActiveBugs();
                Assert.AreEqual(1, actualBugs.Count, "Not 1 bug in collection");
                Assert.AreEqual(bug, actualBugs[0], "Bug objects are not the same");
            }
        }

        [Test]
        public void UpdateBug_InsertBug_UpdateBug_IsActive_ToFalse_ReturnsTrue()
        {
            Bug updatedBug = CreateBug();
            updatedBug.IsActive = false;

            using (_mocks.Record())
                Expect.Call(_repository.Update(updatedBug, null)).IgnoreArguments().Return(true);

            using (_mocks.Playback())
            {
                bool result = _controller.UpdateBug(updatedBug, null);
                Assert.IsTrue(result);
            }
        }
        
        [Test]
        public void InsertBug_NewValidBug_SendEmail()
        {
            using (_mocks.Record())
                Expect.Call(_repository.Add(bug)).Return(true);


            using(_mocks.Playback())
            {
                _controller.InsertBug(bug);
                Assert.AreEqual(1, _emailStub.EmailsSent);
            }
        }

        [Test]
        public void InsertBug_NewValidBug_InsertFailedForSomeReason_NoEmailSent()
        {
            using (_mocks.Record())
                Expect.Call(_repository.Add(bug)).Return(false);


            using (_mocks.Playback())
            {
                _controller.InsertBug(bug);
                Assert.AreEqual(0, _emailStub.EmailsSent);
            }
        }
    }
}